package org.kar.karso.api;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.kar.archidata.annotation.AsyncType;
import org.kar.archidata.dataAccess.DBAccess;
import org.kar.archidata.dataAccess.DataAccess;
import org.kar.archidata.dataAccess.QueryAnd;
import org.kar.archidata.dataAccess.QueryCondition;
import org.kar.archidata.dataAccess.addOnSQL.AddOnManyToMany;
import org.kar.archidata.dataAccess.options.Condition;
import org.kar.archidata.exception.InputException;
import org.kar.archidata.exception.SystemException;
import org.kar.archidata.filter.GenericContext;
import org.kar.archidata.filter.PartRight;
import org.kar.archidata.tools.JWTWrapper;
import org.kar.karso.model.AddUserData;
import org.kar.karso.model.Application;
import org.kar.karso.model.ApplicationSmall;
import org.kar.karso.model.ClientToken;
import org.kar.karso.model.RightDescription;
import org.kar.karso.model.UserAuth;
import org.kar.karso.model.UserLinkApplication;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import jakarta.annotation.security.RolesAllowed;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.PATCH;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.SecurityContext;

@Path("/application")
@Produces(MediaType.APPLICATION_JSON)
public class ApplicationResource {
	private static final Logger LOGGER = LoggerFactory.getLogger(ApplicationResource.class);

	public ApplicationResource() {}

	public List<Long> getUserListOfApplication(final Long userId) {
		final List<Long> out = new ArrayList<>();
		List<UserLinkApplication> links = null;
		try {
			links = DataAccess.getsWhere(UserLinkApplication.class,
					new Condition(new QueryCondition("object1id", "=", userId)));
		} catch (final Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			final String result = "SERVER Internal error";
			LOGGER.error("    result: {}", result);
			return out;
		}
		for (final UserLinkApplication app : links) {
			out.add(app.applicationId);
		}
		return out;
	}

	public List<Long> getListOfUsers(final Long applicationId) {
		final List<Long> out = new ArrayList<>();
		List<UserLinkApplication> links = null;
		try {
			links = DataAccess.getsWhere(UserLinkApplication.class,
					new Condition(new QueryCondition("object2id", "=", applicationId)));
		} catch (final Exception e) {
			e.printStackTrace();
			final String result = "SERVER Internal error";
			LOGGER.error("    result: {}", result);
			return out;
		}
		LOGGER.debug("Find list of user for an application: {}", links);
		for (final UserLinkApplication app : links) {
			out.add(app.userId);
		}
		return out;
	}
	////////////////////////////////////////////////////////////////////////////////////////
	//
	// Generic /application/
	//
	////////////////////////////////////////////////////////////////////////////////////////

	@GET
	@RolesAllowed(value = { "USER", "ADMIN" })
	public List<Application> gets(@Context final SecurityContext sc) throws Exception {
		final GenericContext gc = (GenericContext) sc.getUserPrincipal();
		LOGGER.debug("getApplications");
		// TODO filter with the list of element available in his authorizations ...
		final List<Application> tmp = DataAccess.gets(Application.class);
		if (sc.isUserInRole("ADMIN")) {
			return tmp;
		}
		final List<Long> regular = getUserListOfApplication(gc.userByToken.id);
		final List<Application> out = new ArrayList<>();
		for (final Application app : tmp) {
			if (regular.indexOf(app.id) != -1) {
				out.add(app);
			}
		}
		return out;
	}

	@POST
	@RolesAllowed("ADMIN")
	@Consumes(MediaType.APPLICATION_JSON)
	public Application create(final Application application) throws Exception {
		LOGGER.debug("create new application {}", application);
		// verify login or email is correct:
		if (application.name == null || application.name.length() < 5) {
			throw new InputException("name", "create application (name too small: '" + application.name + "')");
		}
		if (application.redirect == null || application.redirect.length() < 6) {
			throw new InputException("redirect",
					"create application (redirect too small: '" + application.redirect + "')");
		}
		application.id = null;
		application.createdAt = null;
		application.deleted = null;
		application.updatedAt = null;
		final Application tmp = DataAccess.insert(application);
		return get(tmp.id);
	}

	////////////////////////////////////////////////////////////////////////////////////////
	//
	// Generic /application/{id}
	//
	////////////////////////////////////////////////////////////////////////////////////////

	@GET
	@Path("{id}")
	@RolesAllowed("ADMIN")
	public Application get(@PathParam("id") final Long id) throws Exception {
		return DataAccess.get(Application.class, id);
	}

	@PATCH
	@Path("{id}")
	@RolesAllowed("ADMIN")
	@Consumes(MediaType.APPLICATION_JSON)
	public Application patch(@PathParam("id") final Long id, @AsyncType(Application.class) final String jsonRequest)
			throws Exception {
		DataAccess.updateWithJson(Application.class, id, jsonRequest);
		return DataAccess.get(Application.class, id);
	}

	@DELETE
	@Path("{id}")
	@RolesAllowed("ADMIN")
	@Produces(value = MediaType.TEXT_PLAIN)
	public void remove(@Context final SecurityContext sc, @PathParam("id") final long applicationId) throws Exception {
		DataAccess.delete(Application.class, applicationId);
	}

	////////////////////////////////////////////////////////////////////////////////////////
	//
	// Generic /{id}/*
	//
	////////////////////////////////////////////////////////////////////////////////////////

	@GET
	@Path("{id}/users")
	@RolesAllowed(value = { "ADMIN" })
	public List<Long> getApplicationUsers(@PathParam("id") final Long applicationId) throws Exception {
		// special case for SSO: (all user have access on the SSO...).

		LOGGER.debug("Request list of user for an applciation: {}", applicationId);
		return getListOfUsers(applicationId);
	}

	////////////////////////////////////////////////////////////////////////////////////////
	//
	// Generic /application/*
	//
	////////////////////////////////////////////////////////////////////////////////////////

	@GET
	@Path("small")
	@RolesAllowed(value = { "USER", "ADMIN" })
	public List<ApplicationSmall> getApplicationsSmall(@Context final SecurityContext sc) throws Exception {
		final GenericContext gc = (GenericContext) sc.getUserPrincipal();
		LOGGER.debug("getApplications");
		final List<Application> tmp = DataAccess.gets(Application.class);
		final List<Long> regular = getUserListOfApplication(gc.userByToken.id);
		final List<ApplicationSmall> out = new ArrayList<>();
		for (final Application app : tmp) {
			if (regular.indexOf(app.id) != -1) {
				out.add(new ApplicationSmall(app.id, app.name, app.description, app.redirect));
			}
		}
		return out;
	}

	@POST
	@Path("{id}/users")
	@RolesAllowed(value = { "ADMIN" })
	@Consumes(MediaType.APPLICATION_JSON)
	public void addUser(@PathParam("id") final Long applicationId, final AddUserData data) throws Exception {
		LOGGER.debug("getApplications");
		try (DBAccess db = DBAccess.createInterface()) {
			AddOnManyToMany.addLink(db, UserAuth.class, data.userId, "application", applicationId);
		}
	}

	@DELETE
	@Path("{id}/users/${userId}")
	@RolesAllowed(value = { "ADMIN" })
	public void removeUser(@PathParam("id") final Long applicationId, @PathParam("userId") final Long userId)
			throws Exception {
		LOGGER.debug("getApplications");
		try (DBAccess db = DBAccess.createInterface()) {
			AddOnManyToMany.removeLink(db, UserAuth.class, userId, "application", applicationId);
		}
	}

	@GET
	@Path("{id}/rights")
	@RolesAllowed(value = { "ADMIN" })
	public List<RightDescription> getRightsDescription(@PathParam("id") final Long applicationId) throws Exception {
		LOGGER.debug("getApplications rights");
		return DataAccess.getsWhere(RightDescription.class,
				new Condition(new QueryCondition("applicationId", "=", applicationId)));
	}

	@GET
	@Path("get_token")
	@RolesAllowed(value = { "USER", "ADMIN" })
	public ClientToken getClientToken(
			@Context final SecurityContext sc,
			@QueryParam("application") final String application) throws Exception {
		final GenericContext gc = (GenericContext) sc.getUserPrincipal();
		LOGGER.info("get application TOKEN application name='{}'", application);
		LOGGER.debug("== USER ? {}", gc.userByToken.name);

		if (application == null) {
			throw new InputException("application", "Input error missing parameter");
		}
		String applicationName = application;
		boolean isDev = false;
		if (applicationName.endsWith("-dev")) {
			applicationName = applicationName.substring(0, applicationName.length() - 4);
			isDev = true;
		}
		LOGGER.debug("Search for '{}' base of '{}'", applicationName, application);

		Application appl = null;
		try {
			appl = DataAccess.getWhere(Application.class,
					new Condition(new QueryCondition("name", "=", applicationName)));
		} catch (final Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			final String result = "SERVER Internal error";
			LOGGER.debug("    result: {}", result);
			throw new SystemException("Internal fail request");
		}

		if (appl == null) {
			final String result = "Authentiocate-wrong email/login '" + applicationName + "')";
			LOGGER.error("    result: {}", result);
			throw new InputException("email/login", "Authentiocate-wrong email/login '" + applicationName + "'");
		}
		UserLinkApplication links = null;
		try {
			links = DataAccess.getWhere(UserLinkApplication.class,
					new Condition(new QueryAnd(new QueryCondition("object1id", "=", gc.userByToken.id),
							new QueryCondition("deleted", "=", 0), new QueryCondition("object2id", "=", appl.id))));
		} catch (final Exception e) {
			e.printStackTrace();
			throw new SystemException("Fail to insert Link");
		}
		if (links == null) {
			throw new SystemException(
					"Authenticate impossible ==> application not accessible '" + applicationName + "'");
		}
		// Get the USER Right
		final Map<String, PartRight> applicationRight = RightResource.getUserRight(gc.userByToken.id, appl.id);
		if (!applicationRight.containsKey("USER")) {
			// If the USER is not override, the system add by default USER
			applicationRight.put("USER", PartRight.READ_WRITE);
		}
		final Map<String, Integer> rightForFront = new HashMap<>();
		for (final Map.Entry<String, PartRight> entry : applicationRight.entrySet()) {
			rightForFront.put(entry.getKey(), entry.getValue().getValue());
		}
		final Map<String, Object> outRight = new HashMap<>();
		// we set the right in the under map to manage multiple application group right.
		// and in some application user can see other user or all user of the
		// application
		outRight.put(applicationName, rightForFront);
		final String ret = JWTWrapper.generateJWToken(gc.userByToken.id, gc.userByToken.name, "KarAuth",
				applicationName, outRight, -appl.ttl);
		// logger.debug(" ==> generate token: {}", ret);
		String returnAdress = appl.redirect;
		if (isDev) {
			returnAdress = appl.redirectDev;
		}
		return new ClientToken(returnAdress, ret);
	}

	@GET
	@Path("return")
	@RolesAllowed(value = { "USER", "ADMIN" })
	public String logOut(@Context final SecurityContext sc, @QueryParam("application") final String application)
			throws SystemException, InputException {
		LOGGER.debug("=====================================");
		LOGGER.debug("Get log_out()");
		LOGGER.debug("=====================================");
		final GenericContext gc = (GenericContext) sc.getUserPrincipal();
		LOGGER.debug("== USER ? {}", gc.userByToken);

		if (application == null) {
			throw new InputException("application", "Input error missing parameter");
		}
		String applicationName = application;
		boolean isDev = false;
		if (applicationName.endsWith("-dev")) {
			applicationName = applicationName.substring(0, applicationName.length() - 4);
			isDev = true;
		}
		LOGGER.debug("Search for '{}' base of '{}'", applicationName, application);

		Application appl = null;
		try {
			appl = DataAccess.getWhere(Application.class,
					new Condition(new QueryCondition("name", "=", applicationName)));
		} catch (final Exception e) {
			e.printStackTrace();
			throw new SystemException("Fail to log-Out...");
		}
		if (appl == null) {
			throw new SystemException("Authentiocate-wrong email/login '" + applicationName + "'");
		}
		String returnAdress = appl.redirect;
		if (isDev) {
			returnAdress = appl.redirectDev;
		}
		return returnAdress;
	}

}
