package org.kar.karso.api;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.kar.archidata.annotation.AsyncType;
import org.kar.archidata.dataAccess.DataAccess;
import org.kar.archidata.dataAccess.QueryAnd;
import org.kar.archidata.dataAccess.QueryCondition;
import org.kar.archidata.dataAccess.options.Condition;
import org.kar.archidata.filter.PartRight;
import org.kar.karso.model.Right;
import org.kar.karso.model.RightDescription;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import jakarta.annotation.security.RolesAllowed;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.PATCH;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.MediaType;

@Path("/right")
@Produces({ MediaType.APPLICATION_JSON })
public class RightResource {
	private static final Logger LOGGER = LoggerFactory.getLogger(RightResource.class);

	public static List<RightDescription> getApplicationRightDecription(final long applicationId) throws Exception {
		return DataAccess.getsWhere(RightDescription.class,
				new Condition(new QueryCondition("applicationId", "=", applicationId)));
	}

	public static List<Right> getRawUserRight(final long userId, final long applicationId) throws Exception {
		// Formatter:off
		return DataAccess.getsWhere(Right.class,
				new Condition(new QueryAnd(new QueryCondition("applicationId", "=", applicationId),
						new QueryCondition("userId", "=", userId))));
		// Formatter:on
	}

	public static Map<String, PartRight> getUserRight(final long userId, final long applicationId) throws Exception {
		final Map<String, PartRight> out = new HashMap<>();
		final List<RightDescription> rightsDescriptions = getApplicationRightDecription(applicationId);
		LOGGER.trace("Get some descriptions: {} applicationId={}", rightsDescriptions.size(), applicationId);
		if (rightsDescriptions != null && rightsDescriptions.size() != 0) {
			final List<Right> rights = getRawUserRight(userId, applicationId);
			LOGGER.trace("Get some user right: count={} userID={} applicationId={}", rights.size(), userId,
					applicationId);
			LOGGER.trace("Rights:");
			for (final Right elem : rights) {
				LOGGER.trace("    - applId={} rightDescriptionId={} value={}", elem.applicationId,
						elem.rightDescriptionId, elem.value);
			}
			LOGGER.trace("RightDescription:");
			for (final RightDescription description : rightsDescriptions) {
				if (description == null) {
					continue;
				}
				LOGGER.trace("    - id={} key={}", description.id, description.key);
			}
			for (final RightDescription description : rightsDescriptions) {
				if (description == null) {
					continue;
				}
				final Right right = rights.stream().filter(elem -> elem.rightDescriptionId.equals(description.id))
						.findAny().orElse(null);
				if (right != null) {
					out.put(description.key, right.value);
				}
			}
		} else {
			// the application does not manage right with Karso (normal use-case)
			LOGGER.debug("Does not manage Karso right...");
		}
		return out;
	}

	public static void updateUserRight(final long userId, final long applicationId, final Map<String, PartRight> delta)
			throws Exception {
		final List<RightDescription> rightsDescriptions = getApplicationRightDecription(applicationId);
		LOGGER.debug("Get some descriptions: {} applicationId={}", rightsDescriptions.size(), applicationId);
		if (rightsDescriptions == null || rightsDescriptions.size() == 0) {
			throw new IllegalArgumentException("Request change right on an application that does not manage any right");
		}
		final List<Right> rights = getRawUserRight(userId, applicationId);
		LOGGER.debug("Get some user right: count={} userID={}", rights.size(), userId);
		for (final RightDescription description : rightsDescriptions) {
			if (description == null) {
				// TODO: this is a really strange case to manage later...
				continue;
			}
			final PartRight newValue = delta.get(description.key);
			if (newValue == null) {
				//No need to update or create...
				continue;
			}
			final List<Right> allRights = rights.stream().filter(elem -> elem.rightDescriptionId.equals(description.id))
					.toList();
			if (allRights.size() > 1) {
				// special case of error somewhere else ... ==> remove all elements (except the first)
				for (int iii = 1; iii < allRights.size(); iii++) {
					LOGGER.error("Remove starnge element in Right id={} ", allRights.get(iii).id);
					DataAccess.delete(Right.class, allRights.get(iii).id);
				}
			}
			if (allRights.size() == 1) {
				final Right right = allRights.get(0);
				// The value exist, we need to update it
				LOGGER.debug("Request update a knonwn parameter: {} with {}", description.key, newValue);
				right.value = newValue;
				// TODO: maybe need to remove the value instead of set a new one ...
				DataAccess.update(right, right.id, List.of("value"));
			} else {
				// we need to create it
				LOGGER.debug("Request create parameter: {} with {}", description.key, newValue);
				final Right right = new Right();
				right.applicationId = applicationId;
				right.userId = userId;
				right.rightDescriptionId = description.id;
				right.value = newValue;
				DataAccess.insert(right);
			}
		}
	}

	@GET
	@RolesAllowed("ADMIN")
	public List<Right> gets() throws Exception {
		return DataAccess.gets(Right.class);
	}

	@POST
	@RolesAllowed("ADMIN")
	@Consumes(MediaType.APPLICATION_JSON)
	public Right post(final Right request) throws Exception {
		return DataAccess.insert(request);
	}

	@GET
	@Path("{id}")
	@RolesAllowed("ADMIN")
	public static Right get(@PathParam("id") final Long id) throws Exception {
		return DataAccess.get(Right.class, id);
	}

	@PATCH
	@Path("{id}")
	@RolesAllowed("ADMIN")
	@Consumes(MediaType.APPLICATION_JSON)
	public Right patch(@PathParam("id") final Long id, @AsyncType(Right.class) final String jsonRequest)
			throws Exception {
		DataAccess.updateWithJson(Right.class, id, jsonRequest);
		return DataAccess.get(Right.class, id);
	}

	@DELETE
	@Path("{id}")
	@RolesAllowed("ADMIN")
	public void remove(@PathParam("id") final Long id) throws Exception {
		DataAccess.delete(Right.class, id);
	}

}
