package org.kar.karso.api;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.kar.archidata.dataAccess.DBAccess;
import org.kar.archidata.dataAccess.DataAccess;
import org.kar.archidata.dataAccess.QueryAnd;
import org.kar.archidata.dataAccess.QueryCondition;
import org.kar.archidata.dataAccess.addOnSQL.AddOnManyToMany;
import org.kar.archidata.dataAccess.options.AccessDeletedItems;
import org.kar.archidata.dataAccess.options.Condition;
import org.kar.archidata.dataAccess.options.ReadAllColumn;
import org.kar.archidata.exception.FailException;
import org.kar.archidata.exception.SystemException;
import org.kar.archidata.filter.GenericContext;
import org.kar.archidata.filter.PartRight;
import org.kar.archidata.model.GetToken;
import org.kar.archidata.tools.JWTWrapper;
import org.kar.karso.migration.Initialization;
import org.kar.karso.model.Application;
import org.kar.karso.model.ChangePassword;
import org.kar.karso.model.ChangePassword.ChangePasswordChecker;
import org.kar.karso.model.DataGetToken;
import org.kar.karso.model.DataGetToken.DataGetTokenChecker;
import org.kar.karso.model.Right;
import org.kar.karso.model.RightDescription;
import org.kar.karso.model.UserAuth;
import org.kar.karso.model.UserAuth.UserAuthChecker;
import org.kar.karso.model.UserAuthGet;
import org.kar.karso.model.UserCreate;
import org.kar.karso.model.UserCreate.UserCreateChecker;
import org.kar.karso.util.ConfigVariable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonInclude;

import jakarta.annotation.security.PermitAll;
import jakarta.annotation.security.RolesAllowed;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.PATCH;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.SecurityContext;

@Path("/users")
@Produces(MediaType.APPLICATION_JSON)
public class UserResource {
	private static final Logger LOGGER = LoggerFactory.getLogger(UserResource.class);
	static final UserAuthChecker CHECKER = new UserAuthChecker();
	static final DataGetTokenChecker CHECKER_REQUEST_TOKEN = new DataGetTokenChecker();
	static final ChangePasswordChecker CHECKER_CHANGE_PASSWORD = new ChangePasswordChecker();
	static final UserCreateChecker CHECKER_CREATE_USER = new UserCreateChecker();
	public static final String APPLICATION = "karso";

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public static class UserOut {
		public long id;
		public String login;

		public UserOut(final long id, final String login) {
			this.id = id;
			this.login = login;
		}
	}

	public UserResource() {}

	@GET
	@RolesAllowed("ADMIN")
	public List<UserAuthGet> gets() throws Exception {
		final List<UserAuthGet> out = DataAccess.gets(UserAuthGet.class);
		return out;
	}

	@GET
	@Path("{id}")
	@RolesAllowed({ "ADMIN", "USER" })
	public UserAuthGet get(@Context final SecurityContext sc, @PathParam("id") final long id) throws Exception {
		if (!sc.isUserInRole("ADMIN")) {
			// in case of user we need to check if it get his own id parameters:
			final UserAuthGet ret = DataAccess.get(UserAuthGet.class, id, new AccessDeletedItems());
			return ret;
		}
		final UserAuthGet ret = DataAccess.get(UserAuthGet.class, id);
		if (ret != null) {
			return ret;
		}
		// Find element in deleted
		return DataAccess.get(UserAuthGet.class, id, new AccessDeletedItems(), new ReadAllColumn());
	}

	@POST
	@Path("{userId}/application/{applicationId}/link")
	@RolesAllowed("ADMIN")
	public UserAuth linkApplication(
			@Context final SecurityContext sc,
			@PathParam("userId") final long userId,
			@PathParam("applicationId") final long applicationId,
			final boolean data) throws Exception {
		LOGGER.debug("Find typeNode");
		try (DBAccess db = DBAccess.createInterface()) {
			if (data) {
				AddOnManyToMany.addLink(db, UserAuth.class, userId, "application", applicationId);
			} else {
				AddOnManyToMany.removeLink(db, UserAuth.class, userId, "application", applicationId);
			}
			return db.get(UserAuth.class, userId);
		}
	}

	@GET
	@Path("{userId}/application/{applicationId}/rights")
	@RolesAllowed("ADMIN")
	public Map<String, PartRight> getApplicationRight(
			@Context final SecurityContext sc,
			@PathParam("userId") final long userId,
			@PathParam("applicationId") final long applicationId) throws Exception {
		return RightResource.getUserRight(userId, applicationId);
	}

	@PATCH
	@Path("{userId}/application/{applicationId}/rights")
	@RolesAllowed("ADMIN")
	@Consumes(MediaType.APPLICATION_JSON)
	public Map<String, PartRight> patchApplicationRight(
			@Context final SecurityContext sc,
			@PathParam("userId") final long userId,
			@PathParam("applicationId") final long applicationId,
			final Map<String, PartRight> data) throws Exception {
		this.LOGGER.info("Patch data from FRONT: {}", data);
		RightResource.updateUserRight(userId, applicationId, data);
		return RightResource.getUserRight(userId, applicationId);
	}

	// TODO: check this it might be deprecated ...
	@POST
	@Path("{id}/set_admin")
	@RolesAllowed("ADMIN")
	@Consumes(MediaType.APPLICATION_JSON)
	public void setAdmin(@Context final SecurityContext sc, @PathParam("id") final long userId, final boolean data)
			throws Exception {
		try (DBAccess da = DBAccess.createInterface()) {
			final Application appKarso = da.get(Application.class,
					new Condition(new QueryCondition("name", "=", UserResource.APPLICATION)));
			final RightDescription rightsDescription = da.get(RightDescription.class,
					new Condition(new QueryAnd(new QueryCondition("key", "=", "ADMIN"),
							new QueryCondition("applicationId", "=", appKarso.id))));

			final Right right = da.get(Right.class,
					new Condition(new QueryAnd(new QueryCondition("rightDescriptionId", "=", rightsDescription.id),
							new QueryCondition("applicationId", "=", appKarso.id),
							new QueryCondition("userId", "=", userId))));
			if (right == null) {
				final Right newRight = new Right();
				newRight.applicationId = appKarso.id;
				newRight.userId = userId;
				newRight.rightDescriptionId = rightsDescription.id;
				newRight.value = PartRight.READ_WRITE;
				da.insert(newRight);
				return;
			} else if (PartRight.READ_WRITE.equals(right.value)) {
				return;
			}
			// TODO: maybe remove ???
			right.value = PartRight.READ_WRITE;
			final long ret = da.update(right, right.id, List.of("value"));
			if (ret == 0) {
				throw new FailException(Response.Status.NOT_MODIFIED, "Fail to modify user as an admin.");
			}
		}
	}

	@POST
	@Path("{id}/set_blocked")
	@RolesAllowed("ADMIN")
	@Consumes(MediaType.APPLICATION_JSON)
	public void setBlocked(@Context final SecurityContext sc, @PathParam("id") final long userId, final boolean data)
			throws Exception {
		final UserAuth user = new UserAuth();
		user.blocked = data;
		final long ret = DataAccess.update(user, userId, List.of("blocked"));
		if (ret == 0) {
			throw new FailException(Response.Status.NOT_MODIFIED, "Fail to block the User.");
		}
	}

	@POST
	@RolesAllowed("ADMIN")
	@Consumes(MediaType.APPLICATION_JSON)
	public UserAuthGet create(final UserCreate user) throws Exception {
		LOGGER.debug("create new User email={} login={}", user.email, user.login);
		try (DBAccess da = DBAccess.createInterface()) {
			CHECKER_CREATE_USER.checkAll(da, "", user, null);
			// Check login does not exist
			List<UserAuth> out = da.getsWhere(UserAuth.class,
					new Condition(new QueryCondition("login", "=", user.login)));
			if (out.size() >= 1) {
				throw new FailException(Response.Status.BAD_REQUEST, "Login already used !!!");
			}
			// Check email does not exist
			out = da.getsWhere(UserAuth.class, new Condition(new QueryCondition("email", "=", user.email)));
			if (out.size() >= 1) {
				throw new FailException(Response.Status.BAD_REQUEST, "e-mail already used !!!");
			}
			// Add new user and return formated data.
			final UserAuth newUser = new UserAuth();
			newUser.blocked = false;
			newUser.avatar = false;
			newUser.login = user.login;
			newUser.password = user.password;
			newUser.email = user.email;
			newUser.lastConnection = Timestamp.valueOf(LocalDateTime.now());
			final UserAuth tmp = da.insert(newUser);
			LOGGER.debug("create new user done with id=={}", tmp.id);
			return da.get(UserAuthGet.class, tmp.id);
		}
	}

	@GET
	@Path("me")
	@RolesAllowed("USER")
	public UserOut getMe(@Context final SecurityContext sc) {
		LOGGER.debug("getMe()");
		final GenericContext gc = (GenericContext) sc.getUserPrincipal();
		LOGGER.debug("== USER ? {}", gc.userByToken);
		return new UserOut(gc.userByToken.id, gc.userByToken.name);
	}

	@POST
	@Path("password")
	@RolesAllowed("USER")
	@Consumes(MediaType.APPLICATION_JSON)
	public void changePassword(@Context final SecurityContext sc, final ChangePassword data) throws Exception {
		LOGGER.debug("ChangePassword()");
		final GenericContext gc = (GenericContext) sc.getUserPrincipal();
		LOGGER.debug("== USER ? {}", gc.userByToken);
		try (DBAccess da = DBAccess.createInterface()) {
			CHECKER_CHANGE_PASSWORD.checkAll(da, "", data, null);
			final UserAuth user = checkAuthUser(data.login, data.time, data.password);
			if (user == null) {
				throw new SystemException("Fail to retrieve the user... (impossible case)");
			}
			if (!user.id.equals(gc.userByToken.id)) {
				throw new SystemException("Try to change the password of an other user !!! You are BAD...");
			}
			// Process the update:
			user.password = data.newPassword;
			DataAccess.update(user, user.id, List.of("password"));
		}
	}

	@GET
	@Path("is_login_exist")
	@PermitAll
	public Boolean isLoginExist(@QueryParam("login") final String login) throws Exception {
		LOGGER.debug("checkLogin: '{}'", login);
		final List<UserAuth> out = DataAccess.getsWhere(UserAuth.class,
				new Condition(new QueryCondition("login", "=", login)));
		return out.size() >= 1;
	}

	// TODO: add an application TOKEN and permit only 50 requested (maybe add an option to disable it).
	@GET
	@Path("is_email_exist")
	@PermitAll
	public Boolean isEmailExist(@QueryParam("email") final String email) throws Exception {
		LOGGER.debug("checkEmail: {}", email);
		final List<UserAuth> out = DataAccess.getsWhere(UserAuth.class,
				new Condition(new QueryCondition("email", "=", email)));
		return out.size() >= 1;
	}

	private UserAuth checkAuthUser(final String login, final String time, final String password) throws Exception {
		// email or login?
		String query = "login";
		if (login.contains("@")) {
			query = "email";
		}
		final UserAuth user = DataAccess.getWhere(UserAuth.class, new Condition(new QueryCondition(query, "=", login)),
				new AccessDeletedItems(), new ReadAllColumn());

		if (user == null) {
			throw new FailException(Response.Status.PRECONDITION_FAILED,
					"FAIL Authenticate-wrong email/login '" + login + "')");
		}
		// Check the password:
		final String passwordCheck = getSHA512(
				"login='" + login + "';pass='" + user.password + "';date='" + time + "'");
		if (!passwordCheck.contentEquals(password)) {
			throw new FailException(Response.Status.PRECONDITION_FAILED, "Password error ...");
		}
		LOGGER.debug(" ==> pass nearly all test: blocked={} deleted={} ", user.blocked, user.deleted);
		if (user.blocked || (user.deleted != null && user.deleted == true)) {
			throw new FailException(Response.Status.UNAUTHORIZED, "FAIL Authentiocate");
		}
		return user;
	}

	@POST
	@Path("get_token")
	@PermitAll
	@Consumes(MediaType.APPLICATION_JSON)
	public GetToken getToken(final DataGetToken data) throws Exception {
		LOGGER.info("User Authenticate: {}", data.login);
		try (DBAccess da = DBAccess.createInterface()) {
			CHECKER_REQUEST_TOKEN.checkAll(da, "", data, null);
			final UserAuth user = checkAuthUser(data.login, data.time, data.password);
			// at the point the user has been not deleted and not blocked.
			// this authentication is valid only for Karso ==> not for the application
			final int expirationTimeInMinutes = ConfigVariable.getAuthExpirationTime();
			// Get the USER Right (Note: by construction KARSO have application ID = KARSO_INITIALISATION_ID
			final Map<String, PartRight> ssoRight = RightResource.getUserRight(user.id,
					Initialization.KARSO_INITIALISATION_ID);
			if (!ssoRight.containsKey("USER")) {
				// If the USER is not override, the system add by default USER
				ssoRight.put("USER", PartRight.READ_WRITE);
			}
			final Map<String, Integer> rightForFront = new HashMap<>();
			for (final Map.Entry<String, PartRight> entry : ssoRight.entrySet()) {
				rightForFront.put(entry.getKey(), entry.getValue().getValue());
			}
			LOGGER.debug("Get new token with right: {}", ssoRight);
			final Map<String, Object> outRight = new HashMap<>();
			// we set the right in the under map to manage multiple application group right. and in some application user can see other user or all user of the application
			outRight.put(UserResource.APPLICATION, rightForFront);
			// TODO: maybe correct this get of TTL...
			final String ret = JWTWrapper.generateJWToken(user.id, user.login, "KarAuth", UserResource.APPLICATION,
					outRight, expirationTimeInMinutes);
			if (ret == null) {
				throw new SystemException("Missing internal JWT system ==> can not sign anything ...");
			}
			// Update last connection:
			final UserAuth newUser = new UserAuth();
			newUser.lastConnection = Timestamp.valueOf(LocalDateTime.now());
			da.update(newUser, user.id, List.of("lastConnection"));
			return new GetToken(ret);
		}
	}

	public static String bytesToHex(final byte[] bytes) {
		final StringBuilder sb = new StringBuilder();
		for (final byte b : bytes) {
			sb.append(String.format("%02x", b));
		}
		return sb.toString();
	}

	public static String getSHA512(final String passwordToHash) {
		try {
			final MessageDigest md = MessageDigest.getInstance("SHA-512");
			final byte[] bytes = md.digest(passwordToHash.getBytes(StandardCharsets.UTF_8));
			return bytesToHex(bytes);
		} catch (final NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		return null;
	}

}
