package test.kar.karso;

import java.io.IOException;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.kar.archidata.exception.RESTErrorResponseException;
import org.kar.archidata.model.GetToken;
import org.kar.archidata.tools.ConfigBaseVariable;
import org.kar.archidata.tools.RESTApi;
import org.kar.karso.model.DataGetToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
public class TestUnAuthorizedAPI {
	private final static Logger LOGGER = LoggerFactory.getLogger(TestUnAuthorizedAPI.class);

	static WebLauncherTest webInterface = null;
	static RESTApi api = null;

	public void login(final String login, final String password) {
		try {
			final GetToken token = api.post(GetToken.class, "users/get_token", DataGetToken.generate(login, password));
			api.setToken(token.jwt);
		} catch (final Exception ex) {
			Assertions.fail("Can not get Authentication for '" + login + "' ==> " + ex.getMessage());
		}
	}

	public void loginAdmin() {
		login("karadmin", "adminA@666");
	}

	@BeforeAll
	public static void configureWebServer() throws Exception {
		ConfigureDb.configure();
		LOGGER.info("configure server ...");
		webInterface = new WebLauncherTest();
		webInterface.migrateDB();
		LOGGER.info("Start REST (BEGIN)");
		webInterface.process();
		LOGGER.info("Start REST (DONE)");
		api = new RESTApi(ConfigBaseVariable.apiAdress);
	}

	@AfterAll
	public static void stopWebServer() throws InterruptedException, IOException {
		LOGGER.info("Kill the web server");
		webInterface.stop();
		webInterface = null;
		ConfigureDb.clear();
	}

	public void checkFail(final String type, final String urlOffset, final int errorStatus) {
		checkFail(type, urlOffset, errorStatus, null);
	}

	public void checkFail(final String type, final String urlOffset, final int errorStatus, final String data) {
		LOGGER.info("Test API: url={} urlOffset={}", type, urlOffset);
		try {
			if ("GET".equals(type)) {
				api.get(String.class, urlOffset);
			} else if ("POST".equals(type)) {
				api.post(String.class, urlOffset, data);
			} else if ("PATCH".equals(type)) {
				api.patch(String.class, urlOffset, data);
			} else if ("DELETE".equals(type)) {
				api.delete(String.class, urlOffset);
			}
			Assertions.fail("Request on URL does not fail as expected: '" + type + "' url='" + urlOffset + "'");
		} catch (final RESTErrorResponseException ex) {
			if (errorStatus != ex.status) {
				LOGGER.error("Fail in test with the wrong return errors: {}", ex.toString());
			}
			Assertions.assertEquals(errorStatus, ex.status);
		} catch (final Exception ex) {
			LOGGER.error("Unexpected throw error: {}", ex);
			Assertions.fail("Unexpected throws...");
		}

	}

	public void checkWork(final String type, final String urlOffset) {
		checkWork(type, urlOffset, null);
	}

	public void checkWork(final String type, final String urlOffset, final String data) {
		LOGGER.info("Test API: url={} urlOffset={}", type, urlOffset);
		try {
			if ("GET".equals(type)) {
				api.get(String.class, urlOffset);
			} else if ("POST".equals(type)) {
				api.post(String.class, urlOffset, data);
			} else if ("PATCH".equals(type)) {
				api.patch(String.class, urlOffset, data);
			} else if ("DELETE".equals(type)) {
				api.delete(String.class, urlOffset);
			}
			//Assertions.fail("Request on URL does not fail as expected: '" + type + "' url='" + urlOffset + "'");
		} catch (final RESTErrorResponseException ex) {
			Assertions.fail("Must not fail ... " + ex.toString());
		} catch (final Exception ex) {
			LOGGER.error("Unexpected throw error: {}", ex);
			Assertions.fail("Unexpected throws...");
		}

	}

	@Order(1)
	@Test
	public void checkUnAuthorizedAPI() throws Exception {
		// /application/
		checkFail("GET", "application/", 401);
		checkFail("POST", "application/", 401, "{}");
		checkFail("PATCH", "application/", 405, "{}"); // does not exist
		checkFail("DELETE", "application/", 405); // does not exist
		// /application/{id}
		checkFail("GET", "application/0", 401);
		checkFail("PATCH", "application/0", 401, "{}");
		checkFail("POST", "application/0", 405, "{}");
		checkFail("DELETE", "application/0", 401);
		// /application/{id}/*
		checkFail("GET", "application/0/users", 401);
		// /application/*
		checkFail("GET", "application/small", 401);
		checkFail("GET", "application/get_token", 401);
		checkFail("GET", "application/return", 401);

		// /application_token/ section:
		checkFail("GET", "application_token/0", 401);
		checkFail("DELETE", "application_token/0/5", 401);
		checkFail("DELETE", "application_token/0/create", 401);

		// /front/*
		checkFail("GET", "front", 404); // no index in test section
		// health check
		checkWork("GET", "health_check");

		// public_key (only application)
		checkFail("GET", "public_key", 401);
		checkFail("GET", "public_key/pem", 401);

		// /right
		checkFail("GET", "right", 401);
		checkFail("POST", "right", 401, "{}");
		checkFail("GET", "right/0", 401);
		checkFail("PATCH", "right/0", 401, "{}");
		checkFail("DELETE", "right/0", 401);

		// /system_config
		checkWork("GET", "system_config/is_sign_up_availlable");
		checkFail("GET", "system_config/key/skjdfhkjsdhfkjsh", 401);
		checkFail("PATCH", "system_config/key/skjdfhkjsdhfkjsh", 401, "{}");

		// /users
		checkFail("GET", "users", 401);
		checkFail("GET", "users/0", 401);
		checkFail("POST", "users/0/application/0/link", 401, "{}");
		checkFail("POST", "users/0/set_admin", 401, "{}");
		checkFail("POST", "users/0/set_blocked", 401, "{}");
		checkFail("POST", "users", 401, "{}");
		checkFail("GET", "users/me", 401, "{}");
		checkFail("POST", "users/password", 401, "{}");
		checkWork("GET", "users/is_login_exist?login=karadmin");
		checkWork("GET", "users/is_login_exist?login=jhkjhkjh");
		checkWork("GET", "users/is_email_exist?email=admin@admin.ZZZ");
		checkWork("GET", "users/is_email_exist?email=ksjhdkjfhskjdh");
		// not testable : get_token

	}

}
