/** @file
 * @author Edouard DUPIN
 * @copyright 2018, Edouard DUPIN, all right reserved
 * @license PROPRIETARY (see license file)
 */
import { ApplicationResource, ClientToken, Long } from '@/back-api';
import { toaster, toasterAPIError } from '@/components/ui/toaster';
import { useQuery, useQueryCall, useQueryCallProps } from '@/utils/query';

import { useSessionService } from './session';

export namespace ApplicationService {
  export const useGets = () => {
    const { getRestConfig } = useSessionService();
    return useQuery({
      queryFunction: () => {
        return ApplicationResource.gets({
          restConfig: getRestConfig(),
        });
      },
    });
  };
  export const useGetRightsDescription = (id: Long) => {
    const { getRestConfig } = useSessionService();
    return useQuery({
      queryFunction: () => {
        return ApplicationResource.getRightsDescription({
          restConfig: getRestConfig(),
          params: {
            id,
          },
        });
      },
    });
  };

  export const useGetApplicationReturn = (applicationName: string) => {
    const { getRestConfig } = useSessionService();
    return useQuery<ClientToken>({
      queryFunction: () => {
        return ApplicationResource.getClientToken({
          restConfig: getRestConfig(),
          queries: {
            application: applicationName,
          },
        });
      },
    });
  };
  // getApplicationReturn(application: string): Promise < string > {
  // 	return ApplicationResource.logOut({
  // 		restConfig: this.getRestConfig(),
  // 		queries: {
  // 			application
  // 		},
  // 	});
  // }

  export const useGetClientToken = ({
    config,
  }: {
    config?: Omit<useQueryCallProps<ClientToken, any>, 'queryFunction'>;
  }) => {
    const { getRestConfig } = useSessionService();
    const { call, ...rest } = useQueryCall<
      ClientToken,
      { application: string }
    >({
      queryFunction: (data) => {
        return ApplicationResource.getClientToken({
          restConfig: getRestConfig(),
          queries: { ...data },
        });
      },
      ...config,
    });
    return { getClientToken: call, ...rest };
  };

  /*
		getApplicationSpecificToken(application: string): Promise < ClientToken > {
			return ApplicationResource.getClientToken({
				restConfig: this.getRestConfig(),
				queries: {
					application
				}
			});
		}
	
		addUser(id: Long, userId: Long): Promise < void> {
			return ApplicationResource.addUser({
				restConfig: this.getRestConfig(),
				params: {
					id
				},
				data: {
					userId
				}
			});
		}
	
		removeUser(id: Long, userId: Long): Promise < void> {
			return ApplicationResource.removeUser({
				restConfig: this.getRestConfig(),
				params: {
					id,
					userId
				},
			});
		}
	
		getApplicationReturn(application: string): Promise < string > {
			return ApplicationResource.logOut({
				restConfig: this.getRestConfig(),
				queries: {
					application
				},
			});
		}
		getApplicationsSmall(): Promise < ApplicationSmall[] > {
			return ApplicationResource.getApplicationsSmall({
				restConfig: this.getRestConfig(),
			});
		}
	
		getUsers(id: number): Promise < Long[] > {
			return ApplicationResource.getApplicationUsers({
				restConfig: this.getRestConfig(),
				params: {
					id,
				}
			});
		}
		gets(): Promise < Application[] > {
			return ApplicationResource.gets({
				restConfig: this.getRestConfig(),
			});
		}
		get(id: number): Promise < Application > {
			return ApplicationResource.get({
				restConfig: this.getRestConfig(),
				params: {
					id,
				}
			});
		}
		update(id: number, updateState: Application): Promise < Application > {
			return ApplicationResource.patch({
				restConfig: this.getRestConfig(),
				params: {
					id,
				},
				data: updateState
			});
		}
	
		create(name: string, redirect: string): Promise < Application > {
			return ApplicationResource.create({
				restConfig: this.getRestConfig(),
				data: {
					name,
					redirect,
				}
			});
		}
		remove(id: number): Promise < void> {
			return ApplicationResource.remove({
				restConfig: this.getRestConfig(),
				params: {
					id,
				},
			});
		}*/

  export const useTransferApplication = () => {
    const { getRestConfig, clearToken } = useSessionService();
    const transferToApplicationThatRequiredTheSSO = (
      url: string,
      token: string,
      applicationData?: string
    ): void => {
      if (url.slice(-1) === '/') {
        url = url.slice(0, -1);
      }
      let ssoApplicationReturnUrl;
      if (applicationData === undefined) {
        ssoApplicationReturnUrl = `${url}/aG9tZQ/false/`;
      } else {
        ssoApplicationReturnUrl = `${url}/${applicationData}/false/`;
      }
      console.log(`generate in new URL: ${ssoApplicationReturnUrl + token}`);
      window.location.href = ssoApplicationReturnUrl + token;
    };

    const transferToApplication = (
      appName: string,
      applicationData?: string
    ): void => {
      ApplicationResource.getClientToken({
        restConfig: getRestConfig(),
        queries: { application: appName },
      })
        .then((response) => {
          if (!response.url || !response.jwt) {
            toaster.create({
              title: 'Error data',
              description: 'Fail to retrieve data',
              type: 'error',
            });
          } else {
            toaster.create({
              title: 'Request transfer',
              type: 'success',
            });
            transferToApplicationThatRequiredTheSSO(
              response.url,
              response.jwt,
              applicationData
            );
          }
        })
        .catch((error) => {
          toasterAPIError(error);
          if (error.status == 401) {
            // Authentication error ==> auto_disconnect
            clearToken();
            toaster.create({
              title: 'Disconnected by server',
              description: 'Token is no more available',
              type: 'error',
            });
          }
        });
    };

    return { transferToApplication };
  };
}
