import { useCallback, useEffect, useMemo, useState } from 'react';

import { JwtPayload, PartRight, RESTConfig } from '@/back-api';
import { environment, getApiUrl } from '@/environment';
import { useServiceContext } from '@/service/ServiceContext';
import { isBrowser } from '@/utils/layout';
import { parseToken } from '@/utils/sso';

const TOKEN_KEY = 'karso-token-key-storage';

export const getUserToken = () => {
  return localStorage.getItem(TOKEN_KEY);
};
export type RightGroup = 'ADMIN' | 'USER';

export function getRestConfig(): RESTConfig {
  return {
    server: getApiUrl(),
    token: getUserToken() ?? '',
  };
}

export type SessionServiceProps = {
  token?: string;
  isConnected: boolean;
  setToken: (token: string) => void;
  clearToken: () => void;
  login?: string;
  hasReadRight: (group: RightGroup) => boolean;
  hasWriteRight: (group: RightGroup) => boolean;
  getRestConfig: () => RESTConfig;
};

export const useSessionService = (): SessionServiceProps => {
  const { session } = useServiceContext();
  return session;
};

export const useSessionServiceWrapped = (): SessionServiceProps => {
  const [token, setToken] = useState<string | undefined>(
    isBrowser ? (localStorage.getItem(TOKEN_KEY) ?? undefined) : undefined
  );
  const [config, setConfig] = useState<JwtPayload | undefined>(undefined);

  const updateRight = useCallback(() => {
    console.log('Detect a new token...');
    setConfig(undefined);
    if (token === undefined) {
      console.log(`  ==> No User`);
      if (isBrowser) {
        localStorage.removeItem(TOKEN_KEY);
      }
    } else {
      const dataParsedToken = parseToken(token)?.payload;
      console.log(`Get right: ${JSON.stringify(dataParsedToken, null, 2)}`);
      setConfig(dataParsedToken);
      console.log('  ==> Login ... (try to keep right)');
      if (isBrowser) {
        localStorage.setItem(TOKEN_KEY, token);
      }
    }
  }, [localStorage, parseToken, token]);

  const setTokenLocal = useCallback(
    (token: string) => {
      setToken(token);
      updateRight();
    },
    [updateRight, setToken]
  );

  const clearToken = useCallback(() => {
    console.log('Clear Token');
    setToken(undefined);
    updateRight();
  }, [updateRight, setToken]);

  const hasReadRight = useCallback(
    (group: RightGroup) => {
      if (token === undefined) {
        return false;
      }
      const right = config?.right[environment.applName];
      if (right === undefined) {
        return false;
      }
      return [PartRight.READ, PartRight.READ_WRITE].includes(
        right?.[group] ?? 0
      );
    },
    [config]
  );

  const hasWriteRight = useCallback(
    (group: RightGroup) => {
      if (token === undefined) {
        return false;
      }
      const right = config?.right[environment.applName];
      if (right === undefined) {
        return false;
      }
      return [PartRight.READ, PartRight.READ_WRITE].includes(
        right?.[group] ?? 0
      );
    },
    [config]
  );

  const getRestConfig = useCallback((): RESTConfig => {
    return {
      server: getApiUrl(),
      token: token ?? '',
    };
  }, [token]);

  useEffect(() => {
    updateRight();
  }, [updateRight]);

  return {
    token,
    isConnected: token !== undefined,
    setToken: setTokenLocal,
    clearToken,
    login: config?.login,
    hasReadRight,
    hasWriteRight,
    getRestConfig,
  };
};

export const useHasRight = (part: RightGroup) => {
  const { token, hasReadRight, hasWriteRight } = useSessionService();
  const isReadable = useMemo(() => {
    console.log(`get is read for: ${part} ==> ${hasReadRight(part)}`);
    return hasReadRight(part);
  }, [token, hasReadRight, part]);
  const isWritable = useMemo(() => {
    return hasWriteRight(part);
  }, [token, hasWriteRight, part]);
  return { isReadable, isWritable };
};
