/** @file
 * @author Edouard DUPIN
 * @copyright 2018, Edouard DUPIN, all right reserved
 * @license PROPRIETARY (see license file)
 */

import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { PopInService, NotificationService, isString, isNumber, CheckerParameterType, SettingType, SettingsItem } from '@kangaroo-and-rabbit/kar-cw';
import { Application, ApplicationToken } from 'back-api';
import { ApplicationService, ApplicationTokenService, AdminUserService } from 'base/service';

@Component({
	selector: 'application-setting-user-edit',
	templateUrl: './application-user-edit.html',
	styleUrls: ['./application-user-edit.less'],
	changeDetection: ChangeDetectionStrategy.Default,
})
export class ApplicationUserEditScene implements OnInit {
	id: number = undefined;
	users: any[] = undefined;
	notUsers: any[] = undefined;
	applicationName: string = undefined;
	applicationRef: Application = undefined;
	tokens: ApplicationToken[] = [];

	constructor(
		private adminUserService: AdminUserService,
		private applicationService: ApplicationService,
		private applicationTokenService: ApplicationTokenService,
		private activatedRoute: ActivatedRoute,
		private cdr: ChangeDetectorRef,
		private popInService: PopInService,
		private notificationService: NotificationService,
		private router: Router,
	) {
	}
	ngOnInit() {
		this.id = Number(this.activatedRoute.snapshot.paramMap.get('applicationId'));
		const self = this;
		// get local information of the application
		this.applicationService
			.get(this.id)
			.then((response: Application) => {
				console.log(`??? get full response: ${JSON.stringify(response, null, 4)}`);
				self.applicationName = response.name;
				self.applicationRef = response;
			})
			.catch((error: any) => {
				console.log(`??? get ERROR response: ${JSON.stringify(error, null, 4)}`);
			});
		// get all the user connected on it
		this.applicationService
			.getUsers(this.id)
			.then((userListToFilter: number[]) => {
				console.log(`??? get full response: ${JSON.stringify(userListToFilter, null, 4)}`);
				self.adminUserService
					.getUsers()
					.then((response: any[]) => {
						console.log(`All user available for this application: ${JSON.stringify(response, null, 4)}`);
						self.users = [];
						self.notUsers = [];
						for (let iii = 0; iii < response.length; iii++) {
							if (response[iii].blocked === true) {
								continue;
							}
							if (userListToFilter.indexOf(response[iii].id) >= 0) {
								self.users.push(response[iii]);
							} else {
								self.notUsers.push(response[iii]);
							}
						}
					})
					.catch((error: any) => {
						console.log(`??? get ERROR response: ${JSON.stringify(error, null, 4)}`);
					});
			})
			.catch((error: any) => {
				console.log(`??? get ERROR response: ${JSON.stringify(error, null, 4)}`);
			});
	}

	onRemoveApplicationUser(value: boolean, user: any) {
		console.log(`changeState : ${JSON.stringify(value, null, 2)}`);
		const self = this;
		this.applicationService.removeUser(this.id, user.id)
			.then(() => {
				self.notUsers.push(user);
				const index = self.users.indexOf(user, 0);
				if (index > -1) {
					self.users.splice(index, 1);
				}
			})
			.catch((error: any) => {
				console.log(`return ERROR ${JSON.stringify(error, null, 2)}`);
			});
	}
	onAddApplicationUser(value: boolean, user: any) {
		console.log(`changeState : ${JSON.stringify(value, null, 2)}`);
		const self = this;
		this.applicationService.addUser(this.id, user.id)
			.then(() => {
				self.users.push(user);
				const index = self.notUsers.indexOf(user, 0);
				if (index > -1) {
					self.notUsers.splice(index, 1);
				}
			})
			.catch((error: any) => {
				console.log(`return ERROR ${JSON.stringify(error, null, 2)}`);
			});
	}

	formatTimestamp(unix_timestamp: number) {
		return new Date(unix_timestamp).toISOString().replace('T', ' ').replace('Z', ' GMT').replace('.000', '');
	}



	removeApplicationConfirm(token: ApplicationToken) {
		let self = this;
		/*
		this.applicationTokenService.remove(self.application.id, token.id)
			.then(
				() => {
					const index = self.tokens.indexOf(token, 0);
					if (index > -1) {
						self.tokens.splice(index, 1);
						self.cdr.detectChanges();
					}
				}
			).catch(
				(error: any) => {
					self.notificationService.errorRaw(`Fail to delete application token: [${self.application.id}/${token.id}] : ${token.name} ==> ${error}`)
				}
			);
		*/
	}
	confirmDeleteComment: string = undefined;
	confirmDeleteApplicationToken: ApplicationToken = undefined;

	deleteConfirmed() {
		if (this.confirmDeleteApplicationToken !== undefined) {
			this.removeApplicationConfirm(this.confirmDeleteApplicationToken);
			this.confirmDeleteComment = undefined;
			this.confirmDeleteApplicationToken = undefined;
		}
	}


	createTokenMenu: SettingsItem[] = []

	// this permit to clear the input menu...
	configureInput() {
		this.createTokenMenu = [
			{
				type: SettingType.STRING,
				title: 'Token name:',
				placeholder: 'Enter the token name / decription',
				key: 'name',
				value: '',
				checker: (value: CheckerParameterType) => this.checkName(value),
				require: true,
			},
			{
				type: SettingType.NUMBER,
				title: 'Token TTL:',
				placeholder: 'Enter the Time To Lead (in day)',
				key: 'ttl',
				value: '',
				checker: (value: CheckerParameterType) => this.checkTTL(value),
				require: true,
			},
		];
	}

	public createTokenDisabled: number = undefined;
	public dataCreateApplication: object = {};

	/**
	 * Check the application name is available
	 */
	checkName(value: CheckerParameterType): string | undefined {
		if (!isString(value)) {
			return 'Must be a String';
		}
		if (value.length < 3) {
			return 'This name is too small >=3.';
		}
		return undefined;
	}

	/**
	 * Check the redirection have a good form
	 */
	checkTTL(value: CheckerParameterType): string | undefined {
		if (!isNumber(value)) {
			return 'Must be a number';
		}
		const tokenTTL = Number(value);
		if (tokenTTL < 1) {
			return 'Minimum one day';
		}
		return undefined;
	}

	onCreateValueState(value: number) {
		console.log(`changeState : ${JSON.stringify(value, null, 2)}`);
		this.createTokenDisabled = value;
		// we do not change the main ref ==> notify angular that something have change and need to be re-render???
		this.cdr.detectChanges();
	}
	onCreateValueDeltaValues(value: any) {
		console.log(`onDeltaValues : ${JSON.stringify(value, null, 2)}`);
		this.dataCreateApplication = value;
		// we do not change the main ref ==> notify angular that something have change and need to be re-render???
		this.cdr.detectChanges();
	}
	onEditUserRight(_event: any, user: any) {
		this.router.navigate(['application-user-right-edit', this.applicationRef.id, user.id]);
	}


}
