/** @file
 * @author Edouard DUPIN
 * @copyright 2018, Edouard DUPIN, all right reserved
 * @license PROPRIETARY (see license file)
 */

import { ChangeDetectionStrategy, ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { AsyncActionState, CheckerParameterType, NotificationService, PopInService, SettingType, SettingsItem, isString } from '@kangaroo-and-rabbit/kar-cw';
import { Application } from 'back-api';
import { ApplicationService } from 'base/service';

@Component({
	selector: 'application-setting',
	templateUrl: './applications.html',
	styleUrls: ['./applications.less'],
	changeDetection: ChangeDetectionStrategy.Default,
})
export class ApplicationsScene implements OnInit {
	applications: Application[] = [];

	constructor(
		private applicationService: ApplicationService,
		private notificationService: NotificationService,
		private router: Router,
		private popInService: PopInService,
		private cdr: ChangeDetectorRef,
	) {
	}
	ngOnInit() {
		const self = this;
		this.configureInput();
		this.applicationService
			.gets()
			.then((response: Application[]) => {
				console.log(`??? get full response: ${JSON.stringify(response, null, 4)}`);
				self.applications = response;
			})
			.catch((error: any) => {
				console.log(`??? get ERROR response: ${JSON.stringify(error, null, 4)}`);
			});
	}

	onRemoveApplication(_event: any, application: Application) {
		this.confirmDeleteComment = `Delete the application ID: [${application.id}] ${application.name}`;
		this.confirmDeleteApplication = application;
		this.popInService.open('popin-delete-confirm');
	}

	removeApplicationConfirm(application: Application) {
		const self = this;
		this.applicationService.remove(application.id)
			.then(
				() => {
					const index = self.applications.indexOf(application, 0);
					if (index > -1) {
						self.applications.splice(index, 1);
						self.cdr.detectChanges();
					}
				}
			).catch(
				(error: any) => {
					self.notificationService.errorRaw(`Fail to delete application: [${application.id}]: ${application.name} ==> ${error}`)
				}
			);
	}
	confirmDeleteComment: string = undefined;
	confirmDeleteApplication: Application = undefined;

	deleteConfirmed() {
		if (this.confirmDeleteApplication !== undefined) {
			this.removeApplicationConfirm(this.confirmDeleteApplication);
			this.confirmDeleteComment = undefined;
			this.confirmDeleteApplication = undefined;
		}
	}

	onEditApplication(_event: any, application: Application) {
		this.router.navigate(['application-edit', application.id]);
	}
	onEditRightApplication(_event: any, application: Application) {
		this.router.navigate(['application-user-edit', application.id]);
	}


	createApplicationMenu: SettingsItem[] = []

	// this permit to clear the input menu...
	configureInput() {
		this.createApplicationMenu = [
			{
				type: SettingType.STRING,
				title: 'Name:',
				placeholder: 'Enter application name',
				key: 'name',
				value: '',
				checker: (value) => { return this.checkName(value) },
				require: true,
			},
			{
				type: SettingType.STRING,
				title: 'Redirect:',
				placeholder: 'Enter http redirect addresses',
				key: 'redirect',
				value: '',
				checker: (value) => { return this.checkRedirect(value) },
				require: true,
			},
		];
	}

	public validateButtonCreateApplicationDisabled: number = undefined;
	public dataCreateApplication: object = {};

	/**
	 * Check the application name is available
	 */
	checkName(value: CheckerParameterType): string | undefined {
		if (!isString(value)) {
			return 'must be a string';
		}
		console.log(`input value : ${value}`);
		if (value.length < 6) {
			return 'This name is too small >=6.';
		}
		for (let iii = 0; iii < this.applications.length; iii++) {
			const application = this.applications[iii];
			if (application.name === value) {
				return 'This name already exist.';
			}
		}
		return undefined;
	}

	/**
	 * Check the redirection have a good form
	 */
	checkRedirect(value: CheckerParameterType): string | undefined {
		if (!isString(value)) {
			return 'must be a string';
		}
		if (value.length <= 5) {
			return 'This redirect is too small.';
		}
		return undefined;
	}

	createState: string | boolean = undefined;
	/**
	 * Request the creation of a new application.
	 */
	onCreateApplication(): void {
		console.log(`create user:`);
		this.createState = AsyncActionState.LOADING;
		const self = this;
		this.applicationService.create(this.dataCreateApplication['name'], this.dataCreateApplication['redirect'])
			.then(
				(data: Application) => {
					self.createState = AsyncActionState.DONE;
					console.log(`Get new user: ${JSON.stringify(data, null, 2)}`);
					self.applications.push(data);
					self.cdr.detectChanges();
					this.configureInput();
					setTimeout(() => {
						this.createState = undefined;
					}, 3000);
				}
			).catch(
				(error: any) => {
					self.createState = AsyncActionState.FAIL;
					setTimeout(() => {
						self.createState = undefined;
					}, 3000);
				}
			);
	}
	onCreateValueState(value: number) {
		console.log(`changeState : ${JSON.stringify(value, null, 2)}`);
		this.validateButtonCreateApplicationDisabled = value;
		// we do not change the main ref ==> notify angular that something have change and need to be re-render???
		this.cdr.detectChanges();
	}
	onCreateValueDeltaValues(value: any) {
		console.log(`onDeltaValues : ${JSON.stringify(value, null, 2)}`);
		this.dataCreateApplication = value;
		// we do not change the main ref ==> notify angular that something have change and need to be re-render???
		this.cdr.detectChanges();
	}
}
