/** @file
 * @author Edouard DUPIN
 * @copyright 2018, Edouard DUPIN, all right reserved
 * @license PROPRIETARY (see license file)
 */

import { Injectable } from '@angular/core';
import { UserService, SessionService, sha512 } from '@kangaroo-and-rabbit/kar-cw';
import { DataGetToken, GetToken, Long, UserAuthGet, UserResource } from 'back-api';
import { RESTConfig } from 'back-api/rest-tools';
import { environment } from 'environments/environment';

@Injectable()
export class AdminUserService {
	getRestConfig(): RESTConfig {
		return {
			server: environment.server.karso,
			token: this.session.getToken()
		}
	}
	// 0: Not hide password; 1 hide password;
	private identificationVersion: number = 1;

	constructor(
		private userService: UserService,
		private session: SessionService) {
		console.log('Start AdminUserService');
	}
	/**
	 * Log-in with a clear password
	 * @param _login Login of the user (or e-mail)
	 * @param _password clear password to send at the Oauth server
	 * @returns promise on connection
	 */
	login(_login: string, _password: string, rememberMe?: boolean): Promise<boolean> {
		return this.loginSha(_login, sha512(_password), rememberMe);
	}
	private loginSha(login: string, password: string, rememberMe?: boolean): Promise<boolean> {
		// remove the current session interface
		this.userService.removeSession();
		const self = this;
		return new Promise((resolve, reject) => {
			self.getToken(login, password)
				.then((value: any) => {
					console.log(' =====> We connect on the system...');
					self.userService
						.startSession(value.jwt, rememberMe)
						.then(() => {
							resolve(true);
						})
						.catch(error => {
							reject('Can not start session with jwt token ==> (this is bad)');
						});
				})
				.catch(() => {
					console.log('User NOT created');
					reject('Fail to start session !!!');
				});
		});
	}

	getToken(login: string, password: string): Promise<GetToken> {
		console.log(`AuthService.getToken ... '${login}':'${password}'`);

		// TODO: skip this part if the token is valid !!!
		// this is here we need to route to the SSO on external system.

		const currentDate: string = new Date().toISOString();
		let data: DataGetToken;
		// create request:
		if (this.identificationVersion === 1) {
			data = {
				login: login,
				method: 'v1',
				time: currentDate,
				// we mix the password to be sure that it can not be used an other time ...
				password: sha512(`login='${login}';pass='${password}';date='${currentDate}'`),
			};
		} else {
			console.log('AuthService.login ... Wrong method ...');
		}

		console.log(`call users/connect data=${JSON.stringify(data, null, 2)}`);

		return UserResource.getToken({
			restConfig: this.getRestConfig(),
			data
		});
	}
	getUsers(): Promise<UserAuthGet[]> {
		return UserResource.getUsers({
			restConfig: this.getRestConfig(),
		});
	}

	setAdmin(userId: number, state: boolean): Promise<void> {
		return UserResource.setAdmin({
			restConfig: this.getRestConfig(),
			params: {
				id: userId
			},
			data: state,
		});
	}

	setBlocked(userId: number, state: boolean): Promise<void> {
		return UserResource.setBlocked({
			restConfig: this.getRestConfig(),
			params: {
				id: userId
			},
			data: state,
		});
	}

	createUser(email: string, login: string, password: string): Promise<UserAuthGet> {
		return UserResource.create({
			restConfig: this.getRestConfig(),
			data: {
				login,
				email,
				password: sha512(password)
			},
		});
	}

	checkLogin(login: string): Promise<boolean> {
		return UserResource.isLoginExist({
			restConfig: this.getRestConfig(),
			queries: {
				login
			},
		});
	}

	checkEMail(email: string): Promise<boolean> {
		return UserResource.isEmailExist({
			restConfig: this.getRestConfig(),
			queries: {
				email
			},
		});
	}
	retrievePassword(login: string): Promise<void> {
		throw new Error('retrievePassword: Method not implemented.');
	}

	changePassword(oldPassword: string, newPassword: string): Promise<void> {
		const time: string = new Date().toISOString();
		const login: string = this.session.getLogin();
		let method = null;
		let password = null;
		if (this.identificationVersion === 1) {
			method = 'v1';
			password = sha512(`login='${login}';pass='${sha512(oldPassword)}';date='${time}'`);
		} else {
			return new Promise((resolve, reject) => {
				reject(`Internal Fail (contact administrator).`);
			});
		}
		return UserResource.changePassword({
			restConfig: this.getRestConfig(),
			data: {
				method,
				time,
				login,
				password,
				newPassword: sha512(newPassword),
			},
		});
	}

	getApplicationRights(userId: Long, applicationId: Long): Promise<any> {
		return UserResource.getApplicationRight({
			restConfig: this.getRestConfig(),
			params: {
				applicationId,
				userId,
			}
		});
	}

	updateApplicationRights(userId: number, applicationId: number, dataUpdate: { [key: string]: object; }): Promise<any> {
		return UserResource.patchApplicationRight({
			restConfig: this.getRestConfig(),
			params: {
				applicationId,
				userId,
			},
			data: dataUpdate
		});
	}
}
