/** @file
 * @author Edouard DUPIN
 * @copyright 2018, Edouard DUPIN, all right reserved
 * @license PROPRIETARY (see license file)
 */

import { Injectable } from '@angular/core';
import { isUndefined, SessionService } from '@kangaroo-and-rabbit/kar-cw';
import { SystemConfigResource } from 'back-api';
import { RESTConfig } from 'back-api/rest-tools';
import { environment } from 'environments/environment';


class MultipleRequest {
	requestDone: number = 0;
	failResponse = undefined;
	data = {};

	constructor(
		private resolve: any,
		private reject: any,
		private countRequest: number,
	) { }

	add(key: string, value: any): void {
		this.requestDone += 1;
		this.data[key] = value;
		this.checkFinished();
	}
	fail(key: string, error: any): void {
		this.requestDone += 1;
		if (isUndefined(this.failResponse)) {
			this.failResponse = {};
		}
		this.failResponse[key] = error;
		this.checkFinished();
	}
	checkFinished(): void {
		if (this.requestDone < this.countRequest) {
			return;
		}
		// this is the last one:
		if (isUndefined(this.failResponse)) {
			this.resolve(this.data);
		}
		this.reject({
			fail: this.failResponse,
			done: this.data,
		});
	}
}

@Injectable()
export class SettingsService {
	getRestConfig(): RESTConfig {
		return {
			server: environment.server.karso,
			token: this.session.getToken()
		}
	}
	constructor(private session: SessionService) { }
	/**
	 * Get a generic setting (Key value)
	 * @returns a promise of the data value
	 */
	get(key: string): Promise<string | Number | boolean> {
		return new Promise((resolve, reject) => {
			SystemConfigResource.getKey(
				{
					restConfig: this.getRestConfig(),
					params: {
						key
					}
				})
				.then((response: any) => {
					resolve(response.value);
				})
				.catch((error: any) => {
					reject(`return ERROR ${JSON.stringify(error, null, 2)}`);
				});
		});
	}
	/**
	 * Set a generic setting (Key value)
	 * @returns a promise when set is done
	 */
	set(key: string, value: string | Number | boolean): Promise<void> {
		return SystemConfigResource.setKey({
			restConfig: this.getRestConfig(),
			params: {
				key
			},
			data: {
				value: value
			}
		});
	}
	/**
	 * Get a generic setting (Key value)
	 * @returns a promise of the data value
	 */
	gets(keys: string[]): Promise<object> {
		return new Promise((resolve, reject) => {
			let multipleResponse = new MultipleRequest(resolve, reject, keys.length);
			for (let key of keys) {
				this.get(key)
					.then((result: string | Number | boolean) => {
						multipleResponse.add(key, result);
					})
					.catch((error: any) => {
						multipleResponse.fail(key, error);
					});
			}
		});
	}
	/**
	 * Set a generic setting (Key value)
	 * @param[in] data: List of all Key: value to update
	 * @returns a promise when set is done
	 */
	sets(data: object): Promise<object> {
		return new Promise((resolve, reject) => {
			let keys = Object.keys(data);
			let multipleResponse = new MultipleRequest(resolve, reject, keys.length);
			for (let key of keys) {
				this.set(key, data[key])
					.then(() => {
						multipleResponse.add(key, true);
					})
					.catch((error: any) => {
						multipleResponse.fail(key, error);
					});
			}
		});
	}
}
